﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace DataControl
{
    public class AnalysisCompleteEventArgs : EventArgs
    {
        public byte[] Data;
        public int Type;
    }

    /// <summary>
    /// 数据解析类
    /// </summary>
    public class DataAnalysis : IDisposable
    {
        Thread dataAnalysisThread;//数据解析线程
        public Func<int> AnalysisFunc { get; set; }
        
        List<byte> buffer = new List<byte>();
        public List<byte> Buffer
        {
            get
            {
                return buffer;
            }
        }
        object bufferLock = new object();
        #region Buffer操作
        public void RemoveToHeader(byte head)
        {
            lock (bufferLock)
            {
                int index = buffer.FindIndex(c => c == head);
                if (index == -1)
                {
                    buffer.Clear();
                }
                else
                    buffer.RemoveRange(0, index);
            }
        }
        public void Add(byte b)
        {
            lock (bufferLock)
            {
                buffer.Add(b);
            }
        }
        public void AddRange(byte[] data)
        {
            lock (bufferLock)
            {
                buffer.AddRange(data);
            }
        }
        public void RemoveAt(int index)
        {
            lock (bufferLock)
            {
                buffer.RemoveAt(index);
            }
        }
        public void RemoveRange(int index, int count)
        {
            lock (bufferLock)
            {
                buffer.RemoveRange(index, count);
            }
        }
        public void Clear()
        {
            lock (bufferLock)
            {
                buffer.Clear();
            }
        }
        #endregion
        public delegate void AnalysisCompletedHandler(object sender, AnalysisCompleteEventArgs e);//声明了一个Delegate类型（或类指针）
        public event AnalysisCompletedHandler AnalysisCompletedEvent;//声明了一个Delegate对象
        private int scanTime = 10;

        /// <summary>
        /// 线程检测时间
        /// </summary>
        public int ScanTime
        {
            get { return scanTime; }
            set
            {
                if (scanTime != value)
                {
                    if (value < 10)
                        value = 10;
                    scanTime = value;
                }
            }
        }

        bool threadQuitFlag = false;//线程退出标志
        bool threadRunFlag = false;//线程运行标志
        public bool ThreadRunFlag
        {
            get { return threadRunFlag; }
            set
            {
                if (threadRunFlag != value)
                {
                    threadRunFlag = value;
                    Clear();
                }
            }
        }

        public DataAnalysis()
        {
            dataAnalysisThread = new Thread(TheadHander);
            dataAnalysisThread.Start();

        }

        public void Abort()
        {
            threadRunFlag = false;
            threadQuitFlag = true;
            if (dataAnalysisThread != null)
            {
                dataAnalysisThread.Abort();
            }
            Clear();
        }
        /// <summary>
        /// 自定义数据解析函数
        /// </summary>
        /// <returns></returns>
        int CustomAnalysis()
        {
            int result = -1;
            if (this.Buffer.Count >= 10)//数据必须大于10
            {
                if (this.Buffer[0] == 0x10)
                {
                    if (this.Buffer.Count > 16 && this.Buffer[1] == 0x82)//电池信息38个字节+5
                    {
                        #region 包含电池信息
                        if (this.Buffer.Count>=43)//没达到数量，继续攒
                        {
                            if (this.Buffer[42] == 0x03 && this.Buffer[41] == DataAnalysis.XOR_Check(this.Buffer, 0, 41))
                            {
                                AnalysisCompleteEventArgs args = new AnalysisCompleteEventArgs();
                                byte[] tempBuffer = new byte[43];
                                this.Buffer.CopyTo(0, tempBuffer, 0, 43);
                                this.Buffer.RemoveRange(0, 43);
                                args.Data = tempBuffer;
                                args.Type = 3;
                                result = 1;
                                //Console.WriteLine(string.Format("{0}", DataAnalysis.ByteToHexString(tempBuffer)));
                                this.OnAnalysisCompleted(args);
                            }
                            else
                            {
                                this.RemoveAt(0);
                            }
                        }
                        #endregion
                    }
                    else if (this.Buffer.Count >= 16)
                    {
                        if (this.Buffer[15] == 0x03 && this.Buffer[14] == DataAnalysis.XOR_Check(this.Buffer, 0, 14))//16字节协议
                        {
                            AnalysisCompleteEventArgs args = new AnalysisCompleteEventArgs();
                            byte[] tempBuffer = new byte[16];
                            this.Buffer.CopyTo(0, tempBuffer, 0, 16);
                            this.Buffer.RemoveRange(0, 16);
                            args.Data = tempBuffer;
                            args.Type = 1;
                            result = 1;
                            //Console.WriteLine(string.Format("{0}", DataAnalysis.ByteToHexString(tempBuffer)));
                            this.OnAnalysisCompleted(args);
                        }
                        else if (this.Buffer[9] == 0x03 && this.Buffer[8] == DataAnalysis.XOR_Check(this.Buffer, 0, 8))//10字节协议
                        {
                            AnalysisCompleteEventArgs args = new AnalysisCompleteEventArgs();
                            byte[] tempBuffer = new byte[10];
                            this.Buffer.CopyTo(0, tempBuffer, 0, 10);
                            this.Buffer.RemoveRange(0, 10);
                            args.Data = tempBuffer;
                            args.Type = 0;
                            result = 0;
                            //Console.WriteLine(string.Format("{0}", DataAnalysis.ByteToHexString(tempBuffer)));
                            this.OnAnalysisCompleted(args);
                        }
                        else
                        {
                            this.RemoveAt(0);
                        }
                    }
                    else if (this.Buffer.Count >= 10)
                    {
                        if (this.Buffer[9] == 0x03 && this.Buffer[8] == DataAnalysis.XOR_Check(this.Buffer, 0, 8))//10字节协议
                        {
                            AnalysisCompleteEventArgs args = new AnalysisCompleteEventArgs();
                            byte[] tempBuffer = new byte[10];
                            this.Buffer.CopyTo(0, tempBuffer, 0, 10);
                            this.Buffer.RemoveRange(0, 10);
                            args.Data = tempBuffer;
                            args.Type = 0;
                            result = 0;
                            //Console.WriteLine(string.Format("{0}", DataAnalysis.ByteToHexString(tempBuffer)));
                            this.OnAnalysisCompleted(args);
                        }
                        else
                        {
                            this.RemoveAt(0);
                        }
                    }
                }
                else
                {
                    this.RemoveToHeader(0x10);
                }
            }
            return result;
        }

        public int Analysis()
        {
            if (AnalysisFunc != null)
            {
                return AnalysisFunc();
            }
            else
            {
                return CustomAnalysis();
            }
            
        }
        /// <summary>
        /// 数据处理完成后调用的函数
        /// </summary>
        /// <param name="e"></param>
        public void OnAnalysisCompleted(AnalysisCompleteEventArgs e)
        {
            //判断数据处理完成事件是否被创建
            if (AnalysisCompletedEvent != null)
            {
                //数据处理完成后新建立一个线程，并调用回调函数
                Thread thread = new Thread(new ThreadStart(new Action(delegate()
                {
                    AnalysisCompletedEvent(this, e);
                })));
                thread.Start();
            }
        }
        void TheadHander()
        {
            while (!threadQuitFlag)
            {
                if (threadRunFlag)
                {
                    if (buffer.Count > 2048)
                    {
                        Clear();
                    }
                    int type = Analysis();
                }

                Thread.Sleep(ScanTime);
            }
        }

        /// <summary>
        /// 执行与释放或重置非托管资源相关的应用程序定义的任务
        /// </summary>
        public void Dispose()
        {
            Abort();
        }

        /// <summary>
        /// 字节流转换为十六进制字符串
        /// </summary>
        /// <param name="buffer">字节流</param>
        /// <returns>十六进制字符串</returns>
        public static string ByteToHexString(byte[] buffer)
        {
            StringBuilder builder = new StringBuilder(buffer.Length * 3);
            foreach (byte data in buffer)
            {
                builder.Append(Convert.ToString(data, 16).PadLeft(2, '0').PadRight(3, ' '));
            }
            return builder.ToString().ToUpper();
        }

        /// <summary>
        /// 取异或校验码
        /// </summary>
        /// <param name="buf">协议字符串</param>
        /// <param name="index">开始位</param>
        /// <param name="length">协议字符长度</param>
        /// <returns></returns>
        public static byte XOR_Check(byte[] buf, int index, int length)
        {
            byte bcc = 0;
            if (index > length)
                return 0;
            for (int i = index; i < length; i++)
            {
                bcc ^= buf[i];
            }
            return Convert.ToByte(bcc);
        }

        /// <summary>
        /// 取异或校验码
        /// </summary>
        /// <param name="buf">协议字符串</param>
        /// <param name="index">开始位</param>
        /// <param name="length">协议字符长度</param>
        /// <returns>校验码</returns>
        public static byte XOR_Check(List<byte> buf, int index, int length)
        {
            byte bcc = 0;
            if (index > length)
                return 0;
            for (int i = index; i < length; i++)
            {
                bcc ^= buf[i];
            }
            return Convert.ToByte(bcc);
        }
    }
}
