﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using AGV_WPF.Models;
using System.Configuration;
using System.Runtime.Caching;
using Services;
using AGV_WPF_Global;

namespace AGV_WPF.DataControl
{
    public class DataHandler : IDisposable
    {
        Thread dataHandleThread;//数据解析线程
        const int SleepTime=50;

        //帧校验结果
        public const byte VERIFY_NOERROR = 0;
        public const byte VERIFY_HEADERROR = 1;
        public const byte VERIFY_ENDERROR = 2;
        public const byte VERIFY_FUNERROR = 3;
        public const byte VERIFY_BCCERROR = 4;

        //缓存线程操作锁
        object bufferLock = new object(); 

        /// <summary>
        /// 初始化
        /// </summary>
        public DataHandler()
        {
            dataHandleThread = new Thread(TheadHander);
            dataHandleThread.Start();
        }

        #region 线程操作

        /// <summary>
        /// 线程处理函数
        /// </summary>
        void TheadHander()
        {
            while (dataHandleThread.ThreadState != ThreadState.Unstarted)
            {
                if (dataHandleThread.ThreadState == ThreadState.Running)
                {
                    AgvDataAnalysis();
                }
                Thread.Sleep(SleepTime);
            }
        }
        int offLineSet = 1000/SleepTime;//重置连接状态的计算次数：2秒/线程间隔时间
        /// <summary>
        /// 数据解析函数
        /// </summary>
        /// <returns></returns>
        void AgvDataAnalysis()
        {
            //循环获取每台AGV的数据
            for (int i = 0; i < GlobalPara.AgvInfoDic.Count; i++)
            {
                TrafficAreaService trafficService = new TrafficAreaService();
                if (GlobalPara.AgvInfoDic.ContainsKey(i))
                { 
                    AGVInfoModel agvinfo = GlobalPara.AgvInfoDic[i];
                    //if (agvinfo.ConnectStatus)
                    {
                        //通讯每隔一个线程周期就增加一次，按照计算，应该在满2000ms后，主动掉线
                        agvinfo.OffLineCount = agvinfo.OffLineCount + 1;
                        //固定每隔2秒钟重置一下连接状态，设置所有AGV掉线
                        if (agvinfo.OffLineCount > offLineSet) agvinfo.ConnectStatus = false;

                        //得到AGV现在所在管制区号
                        //int newTrafficnum = TrafficAreaService.GetTrafficNumByMark(agvinfo.MARKNUM);

                        //AGV上一个管制区号不为0，表示它上一次的位置是在管制区内
                        if (agvinfo.PTRAFFICNUM > 0)
                        {
                            #region 有旧管制区号的情况
                            //新管制区大于0：表示AGV已经从旧管制区进入新管制区了
                            if (agvinfo.TRAFFICNUM.Count > 0)
                            {
                                #region 由一个管制区进入另一个管制区
                                //if (agvinfo.PTRAFFICNUM != agvinfo.TRAFFICNUM)
                                {
                                    #region 第一步：退出旧管制区，退出队列
                                    if (GlobalPara.TrafficBuffer.Keys.Contains(agvinfo.PTRAFFICNUM))
                                    {
                                        TrafficAreaModel tra = GlobalPara.TrafficBuffer[agvinfo.PTRAFFICNUM];
                                        
                                        if (tra.listAgvQueue.Contains(agvinfo.AGVNUM))
                                        {
                                            
                                            
                                            trafficService.RemoveQueue(agvinfo.PTRAFFICNUM, agvinfo.AGVNUM);
                                        }
                                        if (tra.MainAgv == agvinfo.AGVNUM)
                                        {
                                            //ResetTraffic(agvinfo.TRAFFICNUM);//清空管制区管制状态和主机信息
                                        }
                                        if (tra.IsTraffic) { tra.IsTraffic = false; }

                                    }
                                    #endregion
                                    #region 第二步：进入新管制区,先排队等候放行信号
                                    //if (GlobalPara.TrafficBuffer.Keys.Contains(agvinfo.TRAFFICNUM))
                                    //{
                                    //    if (!GlobalPara.TrafficBuffer[agvinfo.TRAFFICNUM].listAgvQueue.Contains(agvinfo.AGVNUM))
                                    //    {
                                    //        trafficService.AddQueue(agvinfo.TRAFFICNUM, agvinfo.AGVNUM);
                                    //    }
                                    //}
                                    #endregion

                                    //agvinfo.PTRAFFICNUM = agvinfo.TRAFFICNUM;//更新旧管制区号
                                }
                                
                                #endregion
                            }
                            else
                            {
                                #region 由管制区内驶出管制区外
                                //旧管制区不为0，新管制区等于0：表示该AGV由管制区内驶出了管制区外
                                //退出管制区，退出队列
                                if (GlobalPara.TrafficBuffer.Keys.Contains(agvinfo.PTRAFFICNUM))
                                {
                                    TrafficAreaModel tra = GlobalPara.TrafficBuffer[agvinfo.PTRAFFICNUM];
                                    if (tra.listAgvQueue.Contains(agvinfo.AGVNUM))
                                    {
                                        trafficService.RemoveQueue(agvinfo.PTRAFFICNUM, agvinfo.AGVNUM);
                                    }
                                    if (tra.MainAgv == agvinfo.AGVNUM)
                                    {
                                        ResetTraffic(agvinfo.PTRAFFICNUM);//清空管制区管制状态和主机信息
                                    }
                                    if (tra.IsTraffic) { tra.IsTraffic = false; }
                                }
                                //agvinfo.PTRAFFICNUM = agvinfo.TRAFFICNUM;//更新旧管制区号
                                
                                #endregion
                            }
                            #endregion
                        }
                        else//AGV上一个管制区号==0，表示它上一次的位置是不在管制区
                        {
                            #region 没有旧管制区的情况
                            //旧管制区号==0而新管制区号!=0，表示该AGV是由管制区外面进入了管制区
                            //if (agvinfo.TRAFFICNUM!= 0)
                            {
                                #region 进入新管制区,先排队等候放行信号
                                //if (GlobalPara.TrafficBuffer.Keys.Contains(agvinfo.TRAFFICNUM))
                                //{
                                //    if (!GlobalPara.TrafficBuffer[agvinfo.TRAFFICNUM].listAgvQueue.Contains(agvinfo.AGVNUM))
                                //    {
                                //        trafficService.AddQueue(agvinfo.TRAFFICNUM, agvinfo.AGVNUM);
                                //    }
                                //}
                                #endregion
                              
                                //agvinfo.PTRAFFICNUM=agvinfo.TRAFFICNUM;
                            }
                            
                            #endregion
                        }

                    }

                }
            }

            
        }

        /// <summary>
        /// 重置管制区为非管制状态
        /// </summary>
        /// <param name="p">管制区号</param>
        private void ResetTraffic(int trafficNum)
        {
            if (GlobalPara.TrafficBuffer.Keys.Contains(trafficNum))
            {
                GlobalPara.TrafficBuffer[trafficNum].IsTraffic = false;
                GlobalPara.TrafficBuffer[trafficNum].MainAgv = 0;
            }
        }

        /// <summary>
        /// 关闭线程
        /// </summary>
        public void Dispose()
        {
            //throw new NotImplementedException();
            if (dataHandleThread != null)
            {
                dataHandleThread.Abort();
            }
        }

        #endregion

        #region 数据正确性校验
        /// <summary>
        /// 判断一帧数据正确性，0表示没有错误
        /// </summary>
        /// <param name="buf">接收到的byte字节数组</param>
        /// <returns>错误代码</returns>
        public static byte DataCheck(List<byte> buf)
        {
            if ((buf[0]) != 0x10)
            {
                return VERIFY_HEADERROR;  //头错误
            }
            else if ((buf[buf.Count - 1]) != 0x03)
            {
                return VERIFY_ENDERROR;    //尾错误
            }
            else if ((buf[1]) != 0x62 && (buf[1]) != 0x72)  //功能码错误
            {
                return VERIFY_FUNERROR;
            }
            //else if (GetBCC(buf) != buf[buf.Count - 2])
            //{
            //    return VERIFY_BCCERROR;  // 校验错误 
            //}
            else
            {
                return VERIFY_NOERROR;
            }
        }
        /// <summary>
        /// 判断一帧数据正确性，0表示没有错误
        /// </summary>
        /// <param name="buf">接收到的byte字节数组</param>
        /// <returns>错误代码</returns>
        public static byte DataCheck(byte[] buf)
        {
            if ((buf[0]) != 0x10)
            {
                return VERIFY_HEADERROR;  //头错误
            }
            else if ((buf[buf.Length - 1]) != 0x03)
            {
                return VERIFY_ENDERROR;    //尾错误
            }
            else if ((buf[1]) != 0x62 && (buf[1]) != 0x72)  //功能码错误
            {
                return VERIFY_FUNERROR;
            }
            //else if (GetBCC(buf) != buf[buf.Length - 2])
            //{
            //    return VERIFY_BCCERROR;  // 校验错误 
            //}
            else
            {
                return VERIFY_NOERROR;
            }
        }
        /// <summary>
        /// 取异或校验码
        /// </summary>
        /// <param name="buf"></param>
        /// <param name="length"></param>
        /// <returns></returns>
        byte GetBCC(List<byte> buf)   
        {
            int bcc = 0;
            for (int i = 0; i < buf.Count; i++)
            {
                bcc = bcc ^ (buf[i]);
            }
            return Convert.ToByte(bcc);
        }

        #endregion
    }
}
