﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AGV_WPF.Models;
using Services;
using AGV_WPF_Global;
using System.Threading;

namespace AGV_WPF.DataControl
{
    public  class TrafficAreaService
    {
        //通讯类
         CommunicationBase communication = new UdpCommunication(GlobalPara.HostIP, GlobalPara.UdpComPort, 10);
        //AGV运行控制
         byte[] buf_runctl = { 0x10, 0x71, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x30, 0x03 }; 

        #region 初始化

        /// <summary>
        /// 初始化管制区数据
        /// </summary>
        public  void InitTrafficBuffer()
        {
            TrafficAreaService trafficService = new TrafficAreaService();
            //从数据库获取AGVINFO信息
            IList<TrafficAreaModel> listTraffic = trafficService.GetTaffic();
            
            if (GlobalPara.TrafficBuffer.Count > 0)
            {
                //清除管制区的旧数据
                GlobalPara.TrafficBuffer.Clear();
            }
            //存入缓存区，Dictionary
            foreach (var traffic in listTraffic)
            {
               
                GlobalPara.TrafficBuffer.Add(traffic.TrafficNum, traffic);
            }
        }

        /// <summary>
        /// 初始化管制区的地标卡号字典表，缓存表
        /// </summary>
        public void InitTrafficMarkBuffer()
        {
            if (GlobalPara.TrafficMarkBuffer == null)
            {
                GlobalPara.TrafficMarkBuffer = new Dictionary<int, int>();
            }
            if (GlobalPara.TrafficMarkBuffer.Count > 0)
            {
                GlobalPara.TrafficMarkBuffer.Clear();
            }
            GlobalPara.TrafficMarkBuffer = GetTrafficMark();
        }

        #endregion

        #region 对数据库的存储操作，增删改查
        /// <summary>
        /// 新增AGV信息
        /// </summary>
        /// <param name="agv"></param>
        /// <returns></returns>
        public  void Add(TrafficAreaModel area)
        {
            DAL.ZSql sqlAdd = new DAL.ZSql();//数据库访问变量定义
            sqlAdd.Open("Insert into BAS_TRAFFIC(TrafficNum,TrafficName,MainAgv,EnterTime,IsTraffic) VALUES ("
                + area.TrafficNum.ToString() + ",'" + area.TrafficName.ToString() + "'," + area.MainAgv.ToString()+","+area.EnterTime.ToString() + "," + area.IsTraffic.ToString() + ")");
            sqlAdd.Close();

        }

        /// <summary>
        /// 删除AGV信息
        /// </summary>
        /// <param name="agv"></param>
        /// <returns></returns>
        public  bool Delete(TrafficAreaModel area)
        {
            DAL.ZSql sqlDelete = new DAL.ZSql();//数据库访问变量定义
            sqlDelete.Open("delete BAS_TRAFFIC where TrafficNum='" + area.TrafficNum.ToString() + "'");

            sqlDelete.Close();
            if (sqlDelete.rowcount > 0)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 删除AGV信息
        /// </summary>
        /// <param name="agvnum"></param>
        /// <returns></returns>
        public  bool Delete(int areanum)
        {
            DAL.ZSql sqlDelete = new DAL.ZSql();//数据库访问变量定义
            sqlDelete.Open("delete BAS_TRAFFIC where TrafficNum='" + areanum.ToString() + "'");
            
            if (sqlDelete.rowcount > 0)
            {
                sqlDelete.Close();
                return true;
            }
            else
            {
                sqlDelete.Close();
                return false;
            }
        }

        /// <summary>
        /// 更新管制区表信息
        /// </summary>
        /// <param name="agv"></param>
        public  void Update(TrafficAreaModel traffic)
        {
            DAL.ZSql sqlUpdate = new DAL.ZSql();//数据库访问变量定义
            sqlUpdate.Open("Update BAS_TRAFFIC(TrafficNum,TrafficName,MainAgv,EnterTime,IsTraffic) VALUES ("
                + traffic.TrafficNum.ToString() + ",'" + traffic.TrafficName.ToString() + "'," + traffic.MainAgv.ToString() + "," + traffic.IsTraffic.ToString() + ")");
            sqlUpdate.Close();
        }

        /// <summary>
        /// 获取管制区信息数据
        /// </summary>
        /// <param name="agvnum"></param>
        /// <returns></returns>
        public  TrafficAreaModel GetTrafficArea(int areanum)
        {
            DAL.ZSql dtTraffic = new DAL.ZSql();//数据库访问变量定义
            dtTraffic.Open("select TrafficNum,TrafficName,MainAgv,EnterTime,IsTraffic from BAS_TRAFFIC where TrafficNum='" + areanum.ToString() + "'");
            TrafficAreaModel traffic = new TrafficAreaModel(); ;//管制区对象

            traffic.TrafficNum = Convert.ToInt16(dtTraffic.Rows[0]["TrafficNum"]);
            traffic.TrafficName = dtTraffic.Rows[0]["TrafficName"].ToString();
            int agvnum = Convert.ToInt16(dtTraffic.Rows[0]["MainAgv"]);
            traffic.MainAgv = GlobalPara.AgvInfoDic[agvnum];
            traffic.EnterTime = Convert.ToDateTime(dtTraffic.Rows[0]["EnterTime"]);
            traffic.IsTraffic = Convert.ToBoolean(dtTraffic.Rows[0]["IsTraffic"]);

            dtTraffic.Close();
            return traffic;
        }

        /// <summary>
        /// 根据地标卡号返回管制区编号，如果该地标卡不在管制区内，则返回-1
        /// </summary>
        /// <param name="marknum">地标卡号</param>
        /// <returns>管制区号</returns>
        public  int GetTrafficNumByMark(int marknum)
        {
            if (GlobalPara.TrafficMarkBuffer.ContainsKey(marknum))
            {
                return GlobalPara.TrafficMarkBuffer[marknum];
            }
            else
            {
                return 0;
            }
        }

        public  TrafficAreaModel GetTraffic(int marknum)
        {
            TrafficAreaModel trafficObject = null;
            if (GlobalPara.TrafficMarkBuffer.ContainsKey(marknum))
            {
                int trafficNum=GlobalPara.TrafficMarkBuffer[marknum];
                if (trafficNum > 0)
                {
                    if (GlobalPara.TrafficBuffer.Keys.Contains(trafficNum))
                    {
                        trafficObject = GlobalPara.TrafficBuffer[trafficNum];
                    }
                }   
            }
            return trafficObject;
        }

        public TrafficAreaModel GetTraffic(int marknum,int routenum)
        {
            TrafficAreaModel trafficObject = null;
           
            string strsql = "";
            if (routenum > 0)
            {
                strsql = string.Format("SELECT TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where (linenum={0} or linenum=0) and t_mark.mark={1}", routenum, marknum);          
                //strsql = string.Format("SELECT TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where (linenum={0} or linenum=0) and t_mark.mark={1}and t_mark.WorkLine=(select WorkLine from T_WorkLine where CarID={2})", routenum, marknum,agvnum);
            }
            else
            {
                strsql = string.Format("SELECT TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where t_mark.mark={0}", marknum);
            }
            DAL.ZSql sql1 = new DAL.ZSql();
            sql1.Open(strsql);
            int trafficnum1 = 0;
            if (sql1.rowcount > 0)
            {
                trafficnum1 = Convert.ToInt32(sql1.m_table.Rows[0]["TrafficNum"]);
                if (trafficnum1 > 0)
                {
                    if (GlobalPara.TrafficBuffer.Keys.Contains(trafficnum1))
                    {
                        trafficObject = GlobalPara.TrafficBuffer[trafficnum1];
                    }
                }
            }

            return trafficObject;
        }

        public List<TrafficAreaModel> GetTrafficList(int marknum, int routenum,int agvnum)
        {
            List<TrafficAreaModel> list = new List<TrafficAreaModel>();
            string strsql = "";
            if (routenum > 0)
            {
                 strsql = string.Format("SELECT TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where (linenum={0} or linenum=0) and t_mark.mark={1}and t_mark.WorkLine=(select WorkLine from T_WorkLine where CarID={2})", routenum, marknum,agvnum);
            
                //strsql = string.Format("SELECT TrafficNum FROM T_Traffic where (linenum={0} or linenum=0) and mark={1}");
                //strsql = string.Format("SELECT distinct TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where (linenum={0} or linenum=0) and t_mark.mark={1}", routenum, marknum);
            }
            else
            {
                strsql = string.Format("SELECT TrafficNum FROM T_Traffic left join t_mark on markid=t_mark.id where t_mark.mark={0}and t_mark.WorkLine=(select WorkLine from T_WorkLine where CarID={1})", marknum, agvnum);
            }
            DAL.ZSql sql1 = new DAL.ZSql();
            sql1.Open(strsql);

            TrafficAreaModel trafficObject = null;
            for (int i = 0; i < sql1.rowcount; i++)
            {
                int trafficnum = Convert.ToInt32(sql1.m_table.Rows[i]["TrafficNum"]);
                if (trafficnum > 0)
                {
                    if (GlobalPara.TrafficBuffer.Keys.Contains(trafficnum))
                    {
                        trafficObject = GlobalPara.TrafficBuffer[trafficnum];
                        if (trafficObject != null)
                        {
                            list.Add(trafficObject);
                        }
                    }
                }
            }

            return list;
        }
        /// <summary>
        /// 获取所有管制区信息
        /// </summary>
        /// <param name="agvnum"></param>
        /// <returns></returns>
        public  List<TrafficAreaModel> GetALL()
        {
            DAL.ZSql dtTraffic = new DAL.ZSql();//数据库访问变量定义
            dtTraffic.Open("select TrafficNum,TrafficName,MainAgv,EnterTime,IsTraffic from BAS_TRAFFIC");
            List<TrafficAreaModel> listTraffic = new List<TrafficAreaModel>(); //存放所有管制区信息
            TrafficAreaModel traffic = null;//管制区对象
            for (int i = 0; i < dtTraffic.rowcount; i++)
            {
                traffic = new TrafficAreaModel();
                traffic.TrafficNum = Convert.ToInt16(dtTraffic.Rows[i]["TrafficNum"]);
                traffic.TrafficName = dtTraffic.Rows[i]["TrafficName"].ToString();

                int agvnum = Convert.ToInt16(dtTraffic.Rows[i]["MainAgv"]);
                traffic.MainAgv = GlobalPara.AgvInfoDic[agvnum];
                
                if (!string.IsNullOrEmpty(dtTraffic.Rows[i]["EnterTime"].ToString()))
                {
                    traffic.EnterTime = Convert.ToDateTime(dtTraffic.Rows[i]["EnterTime"]);
                }
                else
                {
                    traffic.EnterTime = DateTime.Now;
                }
                traffic.IsTraffic = Convert.ToBoolean(dtTraffic.Rows[i]["IsTraffic"]);

                DAL.ZSql dtTrafficMark = new DAL.ZSql();//数据库访问变量定义
                dtTrafficMark.Open("select MarkNum from BAS_TRAFFIC_MARK where TrafficNum=" + traffic.TrafficNum);


                int marknum = 0;//存放地标号
                for (int k = 0; k< dtTrafficMark.rowcount; k++)
                {
                    try
                    {
                        marknum = Convert.ToInt16(dtTrafficMark.Rows[k]["MarkNum"]);//强制转换成int数据格式，地标号
                    }
                    catch (Exception ex)
                    {
                        throw(ex);
                    }
                    traffic.listMark.Add(marknum);
                }
                
                listTraffic.Add(traffic);
            }
            dtTraffic.Close();
            return listTraffic;
        }

        public List<TrafficAreaModel> GetTaffic()
        {
            DAL.ZSql dtTraffic = new DAL.ZSql();//数据库访问变量定义
            dtTraffic.Open("select TrafficNum,TrafficName,MainAgv,EnterTime,IsTraffic from BAS_TRAFFIC");
            List<TrafficAreaModel> listTraffic = new List<TrafficAreaModel>(); //存放所有管制区信息
            TrafficAreaModel traffic = null;//管制区对象
            for (int i = 0; i < dtTraffic.rowcount; i++)
            {
                traffic = new TrafficAreaModel();
                traffic.TrafficNum = Convert.ToInt16(dtTraffic.Rows[i]["TrafficNum"]);
                traffic.TrafficName = dtTraffic.Rows[i]["TrafficName"].ToString();

                int agvnum = Convert.ToInt16(dtTraffic.Rows[i]["MainAgv"]);
                if (GlobalPara.AgvInfoDic.ContainsKey(agvnum))
                {
                    traffic.MainAgv = GlobalPara.AgvInfoDic[agvnum];
                }
                else
                {
                    traffic.MainAgv = null;
                }

                if (!string.IsNullOrEmpty(dtTraffic.Rows[i]["EnterTime"].ToString()))
                {
                    traffic.EnterTime = Convert.ToDateTime(dtTraffic.Rows[i]["EnterTime"]);
                }
                else
                {
                    traffic.EnterTime = DateTime.Now;
                }
                traffic.IsTraffic = Convert.ToBoolean(dtTraffic.Rows[i]["IsTraffic"]);

                DAL.ZSql dtTrafficMark = new DAL.ZSql();//数据库访问变量定义
                dtTrafficMark.Open("select t.TrafficNum,m.Mark from T_Traffic t left join T_Mark m on (t.MarkID=m.id ) where t.TrafficNum=" + traffic.TrafficNum);


                int marknum = 0;//存放地标号
                for (int k = 0; k < dtTrafficMark.rowcount; k++)
                {
                    try
                    {
                        int.TryParse(dtTrafficMark.Rows[k]["Mark"].ToString(),out marknum);
                        //marknum = Convert.ToInt16(dtTrafficMark.Rows[k]["Mark"]);//强制转换成int数据格式，地标号
                    }
                    catch (Exception ex)
                    {
                        throw (ex);
                    }
                    if (marknum!=0)
                    {
                     traffic.listMark.Add(marknum);
                    }
                }

                listTraffic.Add(traffic);
            }
            dtTraffic.Close();
            return listTraffic;
        }


        /// <summary>
        /// 获取管制区的地标信息
        /// </summary>
        /// <returns></returns>
         Dictionary<int,int> GetTrafficMark()
        {
            DAL.ZSql dtTrafficMark = new DAL.ZSql();//数据库访问变量定义
            dtTrafficMark.Open("select distinct t.TrafficNum,m.Mark as MarkNum from T_Traffic t left join T_Mark m on t.MarkID=m.id");

            Dictionary<int, int> dic = new Dictionary<int, int>();
            int area = 0,mark=0;
            for (int i = 0; i<dtTrafficMark.rowcount; i++)
            {
                area=Convert.ToInt16(dtTrafficMark.Rows[i]["TrafficNum"]);//管制区号
                mark=Convert.ToInt16(dtTrafficMark.Rows[i]["MarkNum"]);//地标卡号
                dic.Add(mark, area);
            }
            dtTrafficMark.Close();
            return dic;
        }

        #endregion

        #region 管制区业务逻辑

        #region 排队队列管理

        /// <summary>
        /// 把AGV插入到从机队列
        /// </summary>
        /// <param name="trafficnum"></param>
        /// <param name="agvnum"></param>
        public  void AddQueue(int trafficnum, int agvnum)
        {
            TrafficAreaModel tam = null;
            AGVInfoModel agvim = null;
            if (GlobalPara.AgvInfoDic.ContainsKey(agvnum))
            {
                agvim = GlobalPara.AgvInfoDic[agvnum];
            }
            if (GlobalPara.TrafficBuffer.ContainsKey(trafficnum))
            {
                tam = GlobalPara.TrafficBuffer[trafficnum];
                //队列操作
                if (GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Count > 0)
                {
                    #region 队列中已经有AGV在排队，我将进入队列末尾排队
                    if (!tam.listAgvQueue.Contains(agvim))
                    {
                        //int lastAgv = GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Last();
                        tam.listAgvQueue.Add(agvim);
                        if (tam.MainAgv != null)
                        {
                            GlobalPara.AgvInfoDic[agvnum].TRAFFICAGV = GlobalPara.TrafficBuffer[trafficnum].MainAgv;
                        }
                        
                        GlobalPara.AgvInfoDic[agvnum].TrafficFlag = true;
                        GlobalPara.AgvInfoDic[agvnum].IsMainAgv = false;
                        
                        
                    }
                    #endregion
                }
                else
                {
                    #region 队列中没有AGV，不用等待，直接进入管制区
                    //GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Add(agvnum);
                    GlobalPara.AgvInfoDic[agvnum].TRAFFICAGV = null;//无AGV管制我
                    GlobalPara.AgvInfoDic[agvnum].TrafficFlag = false;//我不需要被管制
                    GlobalPara.AgvInfoDic[agvnum].IsMainAgv = true;//我是主机
                    #endregion
                }
            }
        }
        
        /// <summary>
        /// 退出排队队列
        /// </summary>
        /// <param name="trafficnum"></param>
        /// <param name="agvnum"></param>
        public  bool RemoveQueue(int trafficnum, int agvnum)
        {
            if (GlobalPara.AgvInfoDic.ContainsKey(agvnum) && GlobalPara.TrafficBuffer.Keys.Contains(trafficnum))
            {
                //获取要退出队列的AGV信息
                AGVInfoModel agv = GlobalPara.AgvInfoDic[agvnum];
                try
                {

                    if (agv.IsMainAgv)
                    {
                        //在队列中删除我自己
                        if (GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Contains(agv))
                        {

                            GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Remove(agv);
                            
                            GlobalPara.TrafficBuffer[trafficnum].IsTraffic = false;
                            GlobalPara.TrafficBuffer[trafficnum].MainAgv = null;
                        }
                    }
                    else
                    {
                        //在队列中删除从机
                        if (GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Contains(agv))
                        {
                            //获取管制我的AGV
                            AGVInfoModel myTrafficAgv = agv.TRAFFICAGV;
                            //获取受我管制的AGV编号
                            int nextAgvIndex = GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.IndexOf(agv) + 1;
                            AGVInfoModel nextAgv = null;
                            if (GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Count > nextAgvIndex)
                            {
                                nextAgv = GlobalPara.TrafficBuffer[trafficnum].listAgvQueue[nextAgvIndex];
                            }
                            //管制区从机队列中是否有该AGV
                            if (nextAgv!=null)
                            {

                                nextAgv.TRAFFICAGV = myTrafficAgv;//传递管制编号给排在我后面的AGV
                                
                            }
                            GlobalPara.TrafficBuffer[trafficnum].listAgvQueue.Remove(agv);//删除该AGV从机

                        }
                    }
                    return true;
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return false;
        }

        #endregion

        #region 放行函数管理

        /// <summary>
        /// 在停止点放行AGV
        /// </summary>
        /// <param name="AgvNum">AGV编号</param>
        public  void ReleaseAgv(int AgvNum)
        {
            sendCtrl(AgvNum, 2);
            sendCtrl(AgvNum, 0);
            
        }
        /// <summary>
        /// 在停止点放行AGV
        /// </summary>
        /// <param name="AgvNum">AGV编号</param>
        public  void ReleaseAgv(AGVInfoModel Agv)
        {
            Resume(Agv);//先发复位命令
            Release(Agv);//再发放行命令

        }
        /// <summary>
        /// 放行AGV命令
        /// </summary>
        /// <param name="agv">AGV对象</param>
        /// <returns></returns>
        public  void Release(AGVInfoModel agv)
        {
            sendCtrl(agv, 0);
        }
        public  void Release(int agvnum)
        {
            sendCtrl(agvnum, 0);
        }
        /// <summary>
        /// 命令AGV恢复/继续行走
        /// </summary>
        /// <param name="agv">AGV对象</param>
        /// <returns></returns>
        public  void Resume(AGVInfoModel agv)
        {
            sendCtrl(agv, 1);
        }
        public  void Resume(int agvnum)
        {
            sendCtrl(agvnum, 1);
        }
        /// <summary>
        /// 命令停止等待
        /// </summary>
        /// <param name="agv">AGV对象</param>
        /// <returns></returns>
        public  void Stop(AGVInfoModel agv)
        {
            sendCtrl(agv, 2);
        }
        public  void Stop(int agvnum)
        {
            sendCtrl(agvnum, 2);
        }
        /// <summary>
        /// 发送控制命令给AGV
        /// </summary>
        /// <param name="agv"></param>
         int sendCtrl(int agvnum,int op)
        {
            buf_runctl[0] = 0x10;
            buf_runctl[1] = 0x71;
            buf_runctl[2] = Convert.ToByte(agvnum);

            if (op >= 0 && op < 8)
            {
                buf_runctl[3] = Convert.ToByte((1 << op) | 0x00);
            }
            else
            {
                buf_runctl[3] = 0x00;
            }
            
            buf_runctl[4] = 0;
            buf_runctl[5] = 0;
            buf_runctl[6] = 0;
            buf_runctl[7] = 0;
            buf_runctl[8] = DrvWlConGetBCC(buf_runctl,8);
            buf_runctl[9] = 0x03;
            if (!communication.IsOpen)
            {
                communication.IsOpen = true;
            }
            //管制区前有AGV排队等候，并放行成功 
            communication.SendData(buf_runctl, null);
            return 1;
        }
        /// <summary>
        /// 发送控制命令给AGV
        /// </summary>
        /// <param name="agv">agv对象</param>
        /// <param name="op">操作类型</param>
        /// <returns></returns>
         int sendCtrl(AGVInfoModel agv, int op)
        {
            buf_runctl[0] = 0x10;
            buf_runctl[1] = 0x71;
            buf_runctl[2] = Convert.ToByte(agv.AGVNUM);

            if (op >= 0 && op < 8)
            {
                buf_runctl[3] = Convert.ToByte((1 << op) | 0x00);
            }
            else
            {
                buf_runctl[3] = 0x00;
            }

            buf_runctl[4] = 0x00;
            buf_runctl[5] = Convert.ToByte(agv.ROUTENUM); ;
            buf_runctl[6] = 0x00;
            buf_runctl[7] = 0x00;
            buf_runctl[8] = DrvWlConGetBCC(buf_runctl, 8);
            buf_runctl[9] = 0x03;
            if (!communication.IsOpen)
            {
                communication.IsOpen = true;
            }
            //发送控制命令
            communication.SendData(buf_runctl, null);
            return 1;
        }
        #endregion

        #endregion

        #region 功能函数
        /// <summary>
        /// 取异或校验码
        /// </summary>
        /// <param name="buf"></param>
        /// <param name="length"></param>
        /// <returns></returns>
         byte DrvWlConGetBCC(byte[] buf, int length)
        {
            int bcc = 0, i = 0;
            for (i = 0; i < length; i++)
            {
                bcc = bcc ^ (buf[i]);
            }
            return Convert.ToByte(bcc);
        }

        /// <summary>
        /// 判断该地标卡号是否是管制卡
        /// </summary>
        /// <param name="marknum"></param>
        /// <returns></returns>
        public  bool IsTrafficMark(int marknum)
        {
            bool rs = GlobalPara.TrafficMarkBuffer.ContainsKey(marknum);
            if (rs)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion
    }
}
